import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;

// TODO: USB eject https://winofsql.jp/VA003334/infoboard_page.php?mid=usb&id=080803163310

public class AlertUSB {
    public final static String VERSION = "20240524";
    public final static int DEV_USB = 1<<0;
    public final static int DEV_DVD = 1<<1;
    public final static String USAGE_MESSAGE =
        "usage: java -jar AlertUSB.jar [options]\n" +
        "/CHECK    : メディアが存在すれば警告する\n" +
        "/USB      : USB が存在する場合の警告動作を行う\n" +
        "/DVD      : DVD が存在する場合の警告動作を行う\n" +
        "/HELP     : ヘルプメッセージを表示する\n" +
        "no option : 環境設定\n" +
        "";
    
    public Display display;
    public Shell shell;
    public Shell dialogShell;
    public Preference preference;
    public Audio audio;
    public Beep beep;
    public Hard hard;
    public Net net;

    public AlertUSB(Display display) {
        this.display = display;
        this.shell = new Shell(display);
        this.dialogShell = new Shell(shell, SWT.SYSTEM_MODAL);
        preference = new Preference(this, shell);
        preference.load();
    }

    public static void main(String[] args) {
    	//SoundAudit.main(args); // デバイスを表示させることで、入手できるデバイスが変化する
        AlertUSB alertUSB = new AlertUSB(new Display());
        if (args.length == 0) {
        	alertUSB.config();
        } else {
            int device = alertUSB.getExistDev(args);
            alertUSB.alert(device);
        }
        System.out.println("AlertUSB finished.");
    }

    /** 環境設定 */
    public void config() {
        preference.open();
    }

    /** メディアの存在を検査する */
    public int getExistDev(String args[]) {
        int device = 0;
        for (String arg : args) {
            arg = arg.toUpperCase();
            if (arg.equals("/CHECK")) {
                if (preference.watchUSB() && existsUSB()) device ^= DEV_USB;
                if (preference.watchDVD() && existsDVD()) device ^= DEV_DVD;
            }
            else if (arg.equals("/USB")) device ^= DEV_USB;
            else if (arg.equals("/DVD")) device ^= DEV_DVD;
            else if (arg.equals("/HELP")) System.out.println(USAGE_MESSAGE);
            else {
                System.out.println("Unknown option : " + arg);
                System.out.println(USAGE_MESSAGE);
            }
        }
        return device;
    }

    public boolean existsUSB() {
        String drive = new DeviceList().getAllDevices(DEV_USB);
        return drive.length() > 0;
    }
 
    public boolean existsDVD() {
        String drive = new DeviceList().getAllDevices(DEV_DVD);
        return drive.length() > 0;
    }
    
    /** 警告動作 */
    public void alert(int device) {
        if (device == 0) return;
        alertConsole(device);
        if (preference.noticeDialog()) alertDialog(device);
        if (preference.noticeAudio())  alertAudio(device);
        if (preference.noticeBeep())   alertBeep(device);
        if (preference.noticeHard())   alertHard(device);
        if (preference.noticeNet())    alertNet(device);
        loop();
        waitForThread();
    }
    
    public void alertConsole(int device) {
        if ((device & DEV_USB) != 0) System.out.println("USB!!!");
        if ((device & DEV_DVD) != 0) System.out.println("DVD!!!");
    }
    
    public void alertDialog(int device) {
        new MyDialog(dialogShell, device).start();
    }
    
    public void alertAudio(int device) {
        int len = preference.noticeSoundLen();
        int cyc = preference.noticeSoundCyc();
        int vol = preference.noticeAudioVol();
        String dev = preference.noticeAudioDev();
        String mix = preference.noticeAudioMix();
        // TODO: USB と DVD を両方警告する
        String file = (device == AlertUSB.DEV_USB) ?
        		preference.noticeAudioUSB():
        		preference.noticeAudioDVD();
        if (file.isEmpty()) audio = new Audio(dev, mix, vol, len/cyc, cyc);  
        else                audio = new Audio(dev, mix, vol, file);
        audio.start();
    }
    
    public void alertBeep(int device) {
        int len = preference.noticeSoundLen();
        int cyc = preference.noticeSoundCyc();
        beep = new Beep(len/cyc, cyc);
        beep.start();
    }
    
    public void alertHard(int device) {
        int len = preference.noticeSoundLen();
        int cyc = preference.noticeSoundCyc();
        hard = new Hard(len/cyc, cyc);
        hard.start();
    }
    
    public void alertNet(int device) {
    	net = new Net(device, preference, null);
        display.syncExec(net); // net.start()はthreadのアクセス違反
        // asyncExec() だと通知動作が遅れる（なぜ？）
    }
    public void alertNet(int device, ResultListener rl) {
    	net = new Net(device, preference, rl);
        display.syncExec(net); // net.start()はthreadのアクセス違反
    }
    
    public String getNetURL(int device) {
        net = new Net(device, preference, null);
        return net.getURL();
    }

    /** ウインドウが閉じられるまでループ */
    public void loop() {
        while (dialogShell.isVisible()) {
            if (!display.readAndDispatch()) display.sleep();
        }
        display.dispose();
    }
    
    /** 音の鳴り終わるのを待つ */
    public void waitForThread() {
        while ((audio != null && audio.isAlive())
             || (beep != null && beep.isAlive())
             || (hard != null && hard.isAlive())) {
//          System.out.println("音声が鳴り終わるのを待っている");
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
