/*
Check if removable device exist.
Returns in errorlevel as ORed bits.
2014/8/28 Nobuyuki Tsuchimura

How to compile:
On Cygwin with mingw-gcc-core package, type
    i686-pc-mingw32-gcc -Wall checkUsbDvd.c -o checkUsbDvd.exe
to obtain Windows (32bit) native executable command.

References:
  "How to list physical disks?"
  http://stackoverflow.com/questions/327718/how-to-list-physical-disks

  "GetLogicalDrives"
  http://msdn.microsoft.com/ja-jp/library/cc429329.aspx
*/

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>

BOOL mediaExists(LPCSTR wszPath) {
    ULARGE_INTEGER freeBytesAvailable;
    ULARGE_INTEGER totalNumberOfBytes;
    ULARGE_INTEGER totalNumberOfFreeBytes;
    BOOL ret = GetDiskFreeSpaceEx(wszPath, &freeBytesAvailable,
                                  &totalNumberOfBytes, &totalNumberOfFreeBytes);
    //printf("%s : %I64d : %d\n", wszPath, totalNumberOfFreeBytes.QuadPart, ret);
    return ret;
}


int getAllDevices(int target) {
    DWORD existDrives = GetLogicalDrives();
    CHAR szDrive[MAX_PATH];
    int i, ret = 0;

    for (i=0; i<26; i++) {
        long bit = (1L<<i);
        if (existDrives & bit) {
            sprintf( szDrive, "%c:\\", 'A'+i );
            if (GetDriveType(szDrive)==target && mediaExists(szDrive)) {
                ret |= bit;
            }
        }
    }
    return ret;
}

int main(int argc, char *argv[]) {
    int i, ret = 0;
    BOOL debug = FALSE;
    BOOL usb   = FALSE;
    BOOL dvd   = FALSE;
    BOOL both  = TRUE;

    for (i=1; i<argc; i++) {
        if        (strcmpi(argv[i],"/USB") == 0) {
            usb = TRUE;
            both = FALSE;
        } else if (strcmpi(argv[i],"/DVD") == 0) {
            dvd = TRUE;
            both = FALSE;
        } else if (strcmpi(argv[i],"/DEBUG") == 0) {
            debug = TRUE;
        } else {
            printf("Unknown option : %s\n", argv[i]);

            printf("usage : checkUsbDvd [/USB] [/DVD] [/DEBUG]\n"
                   "Check if removable device exist.\n"
                   "Returns in errorlevel as ORed bits.\n");
               
        }
    }

    if (debug) printf("Now in debugging mode.\n\n");

    if (both) usb = dvd = TRUE;
    if (usb) {
        i = getAllDevices(DRIVE_REMOVABLE);
        if (debug) printf("USB = %d\n", i);
        ret |= i;
    }
    if (dvd) {
        i = getAllDevices(DRIVE_CDROM);
        if (debug) printf("DVD = %d\n", i);
        ret |= i;
    }
    if (debug) printf("errorlevel = %d\n", ret);
    return ret;
}
